function PPMSAdmin() {
}

PPMSAdmin.prototype = {
    initialize: function() {
        this.username = 'admin';
        this.hostname = window.location.hostname;

        this.setWaitScreen(true);

        this.adminPassword = this.initPasswordFields("adminPassword", "adminSeePassword");
        this.ppmsPassword = this.initPasswordFields("ppmsPassword", "ppmsSeePassword");
        this.initForm();

        this.getApplicationConfiguration()
            .then(this.onGetConfigSuccess.bind(this))
            .catch(this.onGetConfigError.bind(this));
    },

    /**
     * Read the user's configuration file
     * @returns a promise resolved with the configuration file content as an object
     */
     getApplicationConfiguration: function() {
        return new Promise(function(resolve, reject) {
            var xhr = new XMLHttpRequest();
            xhr.addEventListener("readystatechange", function() {
                if(this.readyState === 4) {
                    if (xhr.status === 200) {
                        try {
                            var response = JSON.parse(this.responseText);
                            resolve(response);
                        } catch(e) {
                            reject(e);
                        }
                    } else {
                        reject("Invalid HTTP response status " + xhr.status );
                    }
                }
            });
            console.log("Getting admin.json (you can edit this file with your settings)...");
            xhr.open("GET", "admin.json");
            xhr.send();
        });
    },

    onGetConfigSuccess: function(data) {
        if (data.username) {
            this.username = data.username;
        }
        if (data.hostname) {
            this.hostname = data.hostname;
        }
        this.setWaitScreen(false);
    },

    onGetConfigError: function(error) {
        // Show the alert element
        var alert = document.getElementById("error-config");
        if (alert) {
            alert.classList.remove("d-none");
        }

        var detail = document.getElementById("error-config-detail");
        if (detail) {
            detail.innerText = error;
        }

        // Hide the wait spinner (but keep the veil on the form)
        var spinner = document.getElementById("wait-spinner");
        if (spinner) {
            spinner.classList.add("d-none");
        }
    },

    initPasswordFields: function(idInput, idSee) {
        var input = document.getElementById(idInput);
        var see = document.getElementById(idSee);

        see.addEventListener("click", function() {
            if (input.type === "password") {
                input.type = "text";
                see.setAttribute("class", "fa fa-eye-slash");
            } else {
                input.type = "password";
                see.setAttribute("class", "fa fa-eye");
            }
        });

        return input;
    },

    initForm() {
        var form = document.getElementById("set-password-form");
        form.addEventListener("submit", this.onFormSubmit.bind(this));
    },

    getPasswordInfo: function(ha1, username, password) {
        var passphrase = ha1.toLowerCase();
        var encodedPassword = CryptoJS.AES.encrypt(password, passphrase).toString();

        return {
            username,
            password: encodedPassword,
            signature: CryptoJS.SHA1(passphrase + ':' + username + ':' + encodedPassword).toString().toLowerCase()
        }
    },

    setWaitScreen: function(enabled) {
        var body = document.getElementsByTagName("body");
        if (body && body[0]) {
            if (enabled) {
                body[0].classList.remove("no-wait");
            } else {
                body[0].classList.add("no-wait");
            }
        }
    },

    onFormSubmit: function(event) {
        event.preventDefault();

        var client = IDAL.getClient(this.hostname, this.username, this.adminPassword.value);


        var ha1 = CryptoJS.MD5([this.username, 'Authentication required', this.adminPassword.value].join(':')).toString();
        var payload = this.getPasswordInfo(ha1, 'user', this.ppmsPassword.value);

        this.setWaitScreen(true);
        client.post("/api/auth/set-password", payload)
            .then(this.onSetPasswordSuccess.bind(this))
            .catch(this.onSetPasswordError.bind(this));
    },

    onSetPasswordSuccess: function(result) {
        this.setWaitScreen(false);
        this.showStatusModel("set-pincode-success");
    },

    onSetPasswordError: function(error) {
        var clazz = "set-pincode-error";

        this.setWaitScreen(false);

        if (error.errors && error.errors[0]) {
            switch(error.errors[0].appcode) {
                case 'NOT_AUTHORIZED':
                    clazz = "set-pincode-error-not-authorized";
                    break;
                case 'NETWORK_ERROR':
                    clazz = "set-pincode-error-network";
                    break;
            }
        }

        this.showStatusModel(clazz);
    },

    showStatusModel: function(clazz) {
        $("#statusModal .modal-body").attr('class', "modal-body " + clazz);
        $('#statusModal').modal('show');
    }
}

window.addEventListener("load", function() {
    var app = new PPMSAdmin();
    app.initialize();
});
